/*
  Project: RFID RC522 module, LEDs, LCD1602 i2C module, DS1307 RTC module
  Function: Simple time attendance security control system
*/
//***************************************************************************
#include <SPI.h>//include library code for SPI comms
#include <MFRC522.h>//include library code for RFID
#include <LiquidCrystal_I2C.h> //include library code for LCD1602 I2C module
#include <RTClib.h> //include library code for the RTC
#include <Wire.h>//include library code for the RTC
//***************************************************************************
const int SS_PIN = 10;//RFID SS (SDA) pin attached to Arduino Uno digital pin 10
const int RST_PIN = 9;//RFID RST pin attached to Arduino Uno digital pin 9
const int redLEDpin = 8;//red LED attached to Arduino Uno digital pin 8
const int greenLEDpin = 7;//green LED attached to Arduino Uno digital pin 7
const int buzzerPin = 6;//active piezo buzzer attached to Arduino Uno digital pin 6
LiquidCrystal_I2C lcd(0x3F, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);//initialize the library with the numbers of the interface pins
RTC_DS1307 rtc;//create instance RTC
MFRC522 mfrc522(SS_PIN, RST_PIN);   //create MFRC522 instance.
char daysOfTheWeek[7][12] = {"Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"};
// Define check in time
const int checkInHour = 16;
const int checkInMinute = 50;
//Variable to hold user check in
int userCheckInHour;
int userCheckInMinute;
//***************************************************************************
void verifyCheckIn() {
  if ((userCheckInHour < checkInHour) || ((userCheckInHour == checkInHour) && (userCheckInMinute <= checkInMinute))) {
    digitalWrite(greenLEDpin, HIGH);
    delay(200);
    digitalWrite(greenLEDpin, LOW);
    Serial.println("You are at time!");
    lcd.setCursor(0, 0);
    lcd.print("**You are at time!**");
    delay(2000);
    lcd.clear();
  }
  else {
    digitalWrite(redLEDpin, HIGH);
    delay(200);
    digitalWrite(redLEDpin, LOW);
    Serial.println("You are late!");
    lcd.setCursor(0, 0);
    lcd.print("***You are late!****");
    delay(2000);
    lcd.clear();
  }
}
void setup()
{
  Serial.begin(9600);   //initiate a serial communication
  SPI.begin();         //initiate SPI bus
  mfrc522.PCD_Init(); //initiate MFRC522
  pinMode(redLEDpin, OUTPUT); //sets redLEDpin as OUTPUT
  pinMode(greenLEDpin, OUTPUT);//sets greenLEDpin as OUTPUT
  pinMode(buzzerPin, OUTPUT);//sets buzzerPin as OUTPUT
  // Setup for the RTC
  if (!rtc.begin()) {
    Serial.println("Couldn't find RTC");
    while (1);
  }
  else {
    // following line sets the RTC to the date & time this sketch was compiled
    rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
  }
  if (!rtc.isrunning()) {
    Serial.println("RTC is NOT running!");
  }
  Serial.println("Put your RFID tag close to the reader...");
  Serial.println();
  lcd.begin(20, 4);                            //16X2 lcd display
  lcd.setBacklightPin(3, POSITIVE);           //switch on the backlight
  lcd.setBacklight(HIGH);
  lcd.setCursor(0, 0);                       //setting display position
  lcd.print("       SIMPLE      ");
  lcd.setCursor(0, 1);                       //setting display position
  lcd.print("  TIME ATTENDANCE  ");
  lcd.setCursor(0, 2);                       //setting display position
  lcd.print("   ACCESS CONTROL  ");
  lcd.setCursor(0, 3);                       //setting display position
  lcd.print("       SYSTEM      ");
  delay(2000);
  lcd.clear();
}
void loop()
{
  lcd.setCursor(0, 0);                       //setting display position
  lcd.print("Scan your card...");
  lcd.setCursor(0, 1);                       //setting display position
  lcd.print("Card no:");
  lcd.setCursor(0, 2);                       //setting display position
  lcd.print("Status:");
  lcd.setCursor(0, 3);
  DateTime now = rtc.now();
  lcd.print(now.day(), DEC);
  lcd.print("/");
  lcd.print(now.month(), DEC);
  lcd.print("/");
  lcd.print(now.year(), DEC);
  lcd.print(" ");
  lcd.print(now.hour(), DEC);
  lcd.print(":");
  if (now.minute() < 10) {
    lcd.print("0");
  }
  lcd.print(now.minute(), DEC);
  lcd.print(":");
  if (now.second() < 10) {
    lcd.print("0");
  }
  lcd.print(now.second(), DEC);
  lcd.print(" ");
  lcd.print(daysOfTheWeek[now.dayOfTheWeek()]);
  // Look for new cards
  if ( ! mfrc522.PICC_IsNewCardPresent())
  {
    return;
  }
  // Select one of the cards
  if ( ! mfrc522.PICC_ReadCardSerial())
  {
    return;
  }

  //Show UID on serial monitor
  Serial.print("Card UID :");
  // Sound the buzzer when a card is read
  tone(buzzerPin, 2000);
  delay(100);
  noTone(buzzerPin);
  delay(100);
  String content = "";
  byte letter;
  for (byte i = 0; i < mfrc522.uid.size; i++)
  {
    Serial.print(mfrc522.uid.uidByte[i] < 0x10 ? " 0" : " ");
    Serial.print(mfrc522.uid.uidByte[i], HEX);
    content.concat(String(mfrc522.uid.uidByte[i] < 0x10 ? " 0" : " "));
    content.concat(String(mfrc522.uid.uidByte[i], HEX));

    lcd.setCursor(8, 1);                       //setting display position
    for (byte i = 0; i < mfrc522.uid.size; i++)
    {
      lcd.print(mfrc522.uid.uidByte[i] < 0x10 ? " 0" : " ");
      lcd.print(mfrc522.uid.uidByte[i], HEX);
    }
  }
  Serial.println();
  Serial.print("Status : ");
  content.toUpperCase();
  if (content.substring(1) == "00 40 86 19") //change here the UID of the card/cards that you want to give access
  {
    Serial.println("Authorized");
    
    lcd.setCursor(8, 2);                       //setting display position
    lcd.print("Authorized");
    // Print time on Serial monitor
    Serial.print(now.day(), DEC);
    Serial.print('/');
    Serial.print(now.month(), DEC);
    Serial.print('/');
    Serial.print(now.year(), DEC);
    Serial.print(' ');
    Serial.print(now.hour(), DEC);
    Serial.print(':');
    Serial.print(now.minute(), DEC);
    Serial.print(':');
    Serial.print(now.second(), DEC);
    Serial.print(' ');
    Serial.println(daysOfTheWeek[now.dayOfTheWeek()]);
    // Save check in time;
    userCheckInHour = now.hour();
    userCheckInMinute = now.minute();
    verifyCheckIn();
    delay(200);
  }
  else   {
    Serial.println("No access");
    digitalWrite(redLEDpin, HIGH);
    delay(200);
    digitalWrite(redLEDpin, LOW);
    lcd.setCursor(9, 2);                       //setting display position
    lcd.print("No access");
    // Print time on Serial monitor
    Serial.print(now.day(), DEC);
    Serial.print('/');
    Serial.print(now.month(), DEC);
    Serial.print('/');
    Serial.print(now.year(), DEC);
    Serial.print(' ');
    Serial.print(now.hour(), DEC);
    Serial.print(':');
    Serial.print(now.minute(), DEC);
    Serial.print(':');
    Serial.print(now.second(), DEC);
    Serial.print(' ');
    Serial.println(daysOfTheWeek[now.dayOfTheWeek()]);

    delay(200);
    lcd.clear();
  }
}
